#include "crc.h"

/* Function returns the remainder from a CRC calculation on a char* array of
 * length byte_len. Given in discussion slides.
 */
crc crc8(char* data, size_t data_len)
{
	crc crc1 = data[0];

	size_t i;
	int j;
	for (i = 1; i < data_len; i++) {
		char next_byte = data[i];
		/* Start at most significant bit of next byte and work our way down */
		for (j = 7; j >= 0; j--) {
			if ((crc1 & CHAR_MSB) == 0) {
				crc1 <<= 1;		      /* Shift out 0 */
				crc1 |= GET_BIT(j, next_byte); /* Shift in next bit */
			}
			else { /* crc’s most significant bit is a 1
				* Do first 8 bits of modulo 2 subtraction
				*/
				crc1 ^= CRCPOLY;
				crc1 <<= 1;
				/* Do final bit of modulo 2 subtraction and
				 * place in position 0 of crc register
				 */
				crc1 |= (GET_BIT(j, next_byte) ^ 1);
			}
		}
	}
	return crc1;
}

/* This should be done LAST before pickling.
 * Checksum all the contents of the frame EXCEPT for the parity field.
 */
crc calc_frame_crc(Frame *frame)
{
	char *pickled_frame = convert_frame_to_char(frame);
	crc crc1 = crc8(pickled_frame, MAX_FRAME_SIZE - sizeof(crc));
	free(pickled_frame);
	return crc1;
}

/* Frame is corrupted if crc of the contents do not agree with the stored crc.
 */
int is_frame_corrupted(Frame *frame)
{
	crc crc1 = calc_frame_crc(frame);
	DEBUG_PRINT("frame parity: %x  crc: %x\n",
		    frame->parity & 0xff, crc1 & 0xff);
	return (frame->parity != crc1);
}
